use std::{
    env,
    fs::File,
    io::{BufReader, BufWriter},
    path::PathBuf,
};

use rbx_dom_weak::types::Ref;

fn main() {
    let args: Vec<String> = env::args().collect();
    let in_fn = args.get(1).expect("No input file specified");
    let in_file = File::open(in_fn).expect("Failed to open input file");
    let reader = BufReader::new(in_file);

    let mut in_pb = PathBuf::from(in_fn);

    match in_pb.extension() {
        Some(in_ext) => match in_ext.to_str().unwrap() {
            "rbxmx" => {
                let dom = rbx_xml::from_reader_default(reader).expect("Failed to decode file");

                in_pb.set_extension("rbxm");
                let out_fn = in_pb.to_str().unwrap();
                let out_file = File::options()
                    .create_new(true)
                    .write(true)
                    .open(out_fn)
                    .expect("Failed to open output file");
                let writer = BufWriter::new(out_file);

                let model_root = dom.root_ref();

                let mut dm_removed: Vec<Ref> = vec![];
                for child in dom.get_by_ref(model_root).unwrap().children() {
                    dm_removed.push(*child);
                }

                rbx_binary::to_writer(writer, &dom, &dm_removed)
                    .expect("Failed to write output file");
            }
            "rbxm" => {
                let dom = rbx_binary::from_reader(reader).expect("Failed to decode file");

                in_pb.set_extension("rbxmx");
                let out_fn = in_pb.to_str().unwrap();
                let out_file = File::options()
                    .create_new(true)
                    .write(true)
                    .open(out_fn)
                    .expect("Failed to open output file");
                let writer = BufWriter::new(out_file);

                let model_root = dom.root_ref();

                let mut dm_removed: Vec<Ref> = vec![];
                for child in dom.get_by_ref(model_root).unwrap().children() {
                    dm_removed.push(*child);
                }

                rbx_xml::to_writer_default(writer, &dom, &dm_removed)
                    .expect("Failed to write output file");
            }
            &_ => panic!("Not a Roblox model file"),
        },
        None => panic!("Not a Roblox model file"),
    }
}
